@@ -107,8 +107,12 @@ class PDBParser:
 
     def fix_pdbline(self, pdbline, lastnum):
         """Fix a PDB line if information is missing."""
+        pdbqt_conversion = {
+        "HD": "H", "HS": "H", "NA": "N",
+        "NS": "N", "OA": "O", "OS": "O", "SA": "S"}
         fixed = False
         newnum = 0
+        forbidden_characters = "[^a-zA-Z0-9_]"
         pdbline = pdbline.strip('\n')
         # Some MD / Docking tools produce empty lines, leading to segfaults
         if len(pdbline.strip()) == 0:
@@ -123,6 +127,18 @@ class PDBParser:
         if pdbline.startswith('ATOM'):
             newnum = lastnum + 1
             currentnum = int(pdbline[6:11])
+            resnum = pdbline[22:27].strip()
+            resname = pdbline[17:21].strip()
+            # Invalid residue number
+            try:
+                int(resnum)
+            except ValueError:
+                pdbline = pdbline[:22] + '   0 ' + pdbline[27:]
+                fixed = True
+            # Invalid characters in residue name
+            if re.match(forbidden_characters, resname.strip()):
+                pdbline = pdbline[:17] + 'UNK ' + pdbline[21:]
+                fixed = True
             if lastnum + 1 != currentnum:
                 pdbline = pdbline[:6] + (5 - len(str(newnum))) * ' ' + str(newnum) + ' ' + pdbline[12:]
                 fixed = True
@@ -133,14 +149,19 @@ class PDBParser:
             if pdbline.endswith('H'):
                 self.num_fixed_lines += 1
                 return None, lastnum
+            # Sometimes, converted PDB structures contain PDBQT atom types. Fix that.
+            for pdbqttype in pdbqt_conversion:
+                if pdbline.strip().endswith(pdbqttype):
+                    pdbline = pdbline.strip()[:-2] + ' ' + pdbqt_conversion[pdbqttype] + '\n'
+                    self.num_fixed_lines += 1
         if pdbline.startswith('HETATM'):
             newnum = lastnum + 1
             currentnum = int(pdbline[6:11])
             if lastnum + 1 != currentnum:
                 pdbline = pdbline[:6] + (5 - len(str(newnum))) * ' ' + str(newnum) + ' ' + pdbline[12:]
                 fixed = True
-            # No chain assigned
-            if pdbline[21] == ' ':
+            # No chain assigned or number assigned as chain
+            if pdbline[21] == ' ' or re.match("[0-9]", pdbline[21]):
                 pdbline = pdbline[:21] + 'Z' + pdbline[22:]
                 fixed = True
             # No residue number assigned
@@ -148,8 +169,11 @@ class PDBParser:
                 pdbline = pdbline[:23] + '999' + pdbline[26:]
                 fixed = True
             # Non-standard Ligand Names
-            ligname = pdbline[17:20]
-            if re.match("[^a-zA-Z0-9_]", ligname.strip()):
+            ligname = pdbline[17:21].strip()
+            if len(ligname) > 3:
+                pdbline = pdbline[:17] + ligname[:3] + ' ' + pdbline[21:]
+                fixed = True
+            if re.match(forbidden_characters, ligname.strip()):
                 pdbline = pdbline[:17] + 'LIG ' + pdbline[21:]
                 fixed = True
             if len(ligname.strip()) == 0:
@@ -158,6 +182,11 @@ class PDBParser:
             if pdbline.endswith('H'):
                 self.num_fixed_lines += 1
                 return None, lastnum
+            # Sometimes, converted PDB structures contain PDBQT atom types. Fix that.
+            for pdbqttype in pdbqt_conversion:
+                if pdbline.strip().endswith(pdbqttype):
+                    pdbline = pdbline.strip()[:-2] + ' ' + pdbqt_conversion[pdbqttype] + ' '
+                    self.num_fixed_lines +=1
         self.num_fixed_lines += 1 if fixed else 0
         return pdbline + '\n', max(newnum, lastnum)
 
@@ -182,41 +211,78 @@ class LigandFinder:
         self.ligands = self.getligs()
         self.excluded = sorted(list(self.lignames_all.difference(set(self.lignames_kept))))
 
+    def getpeptides(self, chain):
+        """If peptide ligand chains are defined via the command line options,
+        try to extract the underlying ligand formed by all residues in the
+        given chain without water
+        """
+        data = namedtuple('ligand', 'mol hetid chain position water members longname type atomorder can_to_pdb')
+        all_from_chain = [o for o in pybel.ob.OBResidueIter(self.proteincomplex.OBMol) if o.GetChain() == chain] # All residues from chain
+        if len(all_from_chain) == 0:
+            return None
+        else:
+            #TODO Can be done shorter with a split
+            water = [o for o in all_from_chain if o.GetResidueProperty(9)]
+            non_water = water = [o for o in all_from_chain if not o.GetResidueProperty(9)]
+            ligand = self.extract_ligand(non_water)
+            return ligand
+
+
     def getligs(self):
         """Get all ligands from a PDB file and prepare them for analysis.
         Returns all non-empty ligands.
         """
-        ligands = []
 
-        # Filter for ligands using lists
-        ligand_residues, self.lignames_all, self.water = self.filter_for_ligands()
+        if config.PEPTIDES == [] and config.INTRA is None:
+            # Extract small molecule ligands (default)
+            ligands = []
 
-        all_res_dict = {(a.GetName(), a.GetChain(), a.GetNum()): a for a in ligand_residues}
-        self.lignames_kept = list(set([a.GetName() for a in ligand_residues]))
+            # Filter for ligands using lists
+            ligand_residues, self.lignames_all, self.water = self.filter_for_ligands()
+
+            all_res_dict = {(a.GetName(), a.GetChain(), a.GetNum()): a for a in ligand_residues}
+            self.lignames_kept = list(set([a.GetName() for a in ligand_residues]))
+
+            if not config.BREAKCOMPOSITE:
+                #  Update register of covalent links with those between DNA/RNA subunits
+                self.covalent += nucleotide_linkage(all_res_dict)
+                #  Find fragment linked by covalent bonds
+                res_kmers = self.identify_kmers(all_res_dict)
+            else:
+                res_kmers = [[a, ] for a in ligand_residues]
+            for kmer in res_kmers:  # iterate over all ligands and extract molecules + information
+                ligands.append(self.extract_ligand(kmer))
 
-        if not config.BREAKCOMPOSITE:
-            #  Update register of covalent links with those between DNA/RNA subunits
-            self.covalent += nucleotide_linkage(all_res_dict)
-            #  Find fragment linked by covalent bonds
-            res_kmers = self.identify_kmers(all_res_dict)
         else:
-            res_kmers = [[a, ] for a in ligand_residues]
-        for kmer in res_kmers:  # iterate over all ligands and extract molecules + information
-            ligands.append(self.extract_ligand(kmer))
+            # Extract peptides from given chains
+            self.water = [o for o in pybel.ob.OBResidueIter(self.proteincomplex.OBMol) if o.GetResidueProperty(9)]
+            if config.PEPTIDES != []:
+                peptide_ligands = [self.getpeptides(chain) for chain in config.PEPTIDES]
+            elif config.INTRA is not None:
+                peptide_ligands = [self.getpeptides(config.INTRA), ]
+
+            ligands = [p for p in peptide_ligands if p is not None ]
+            self.covalent, self.lignames_kept, self.lignames_all = [], [], set()
+
         return [lig for lig in ligands if len(lig.mol.atoms) != 0]
 
     def extract_ligand(self, kmer):
         """Extract the ligand by copying atoms and bonds and assign all information necessary for later steps."""
         data = namedtuple('ligand', 'mol hetid chain position water members longname type atomorder can_to_pdb')
         members = [(res.GetName(), res.GetChain(), int32_to_negative(res.GetNum())) for res in kmer]
-        members = sorted(members, key=lambda x: (x[1], x[2]))
+        members = sort_members_by_importance(members)
         rname, rchain, rnum = members[0]
         write_message("Finalizing extraction for ligand %s:%s:%s\n" % (rname, rchain, rnum), mtype='debug')
         names = [x[0] for x in members]
         longname = '-'.join([x[0] for x in members])
 
-        # Classify a ligand by its HETID(s)
-        ligtype = classify_by_name(names)
+        if config.PEPTIDES != []:
+            ligtype = 'PEPTIDE'
+        elif config.INTRA is not None:
+            ligtype = 'INTRA'
+        else:
+            # Classify a ligand by its HETID(s)
+            ligtype = classify_by_name(names)
 
         hetatoms = set()
         for obresidue in kmer:
@@ -275,29 +341,37 @@ class LigandFinder:
         return ligand
 
     def is_het_residue(self, obres):
-        """Given an OBResidue, determines if the residue is indeed a ligand
-        in the PDB file (any atoms has to be a HETATM entry)"""
-        het_atoms = []
-        for atm in pybel.ob.OBResidueAtomIter(obres):
-                het_atoms.append(obres.IsHetAtom(atm))
-        if True in het_atoms:
+        """Given an OBResidue, determines if the residue is indeed a possible ligand
+        in the PDB file"""
+        if not obres.GetResidueProperty(0):
+            # If the residue is NOT amino (0)
+            # It can be amino_nucleo, coenzme, ion, nucleo, protein, purine, pyrimidine, solvent
+            # In these cases, it is a ligand candidate
             return True
         else:
-            return False
+            # Here, the residue is classified as amino
+            # Amino acids can still be ligands, so we check for HETATM entries
+            # Only residues with at least one HETATM entry are processed as ligands
+            het_atoms = []
+            for atm in pybel.ob.OBResidueAtomIter(obres):
+                    het_atoms.append(obres.IsHetAtom(atm))
+            if True in het_atoms:
+                return True
+        return False
 
 
     def filter_for_ligands(self):
         """Given an OpenBabel Molecule, get all ligands, their names, and water"""
 
-        #candidates1 = [o for o in pybel.ob.OBResidueIter(self.proteincomplex.OBMol) if not (o.GetResidueProperty(9)
-        #                                                                                or o.GetResidueProperty(0))]
-
         candidates1 = [o for o in pybel.ob.OBResidueIter(self.proteincomplex.OBMol) if not o.GetResidueProperty(9) and self.is_het_residue(o)]
         all_lignames = set([a.GetName() for a in candidates1])
 
         water = [o for o in pybel.ob.OBResidueIter(self.proteincomplex.OBMol) if o.GetResidueProperty(9)]
         # Filter out non-ligands
-        candidates2 = [a for a in candidates1 if is_lig(a.GetName()) and a.GetName() not in self.modresidues]
+        if not config.KEEPMOD:  # Keep modified residues as ligands
+            candidates2 = [a for a in candidates1 if is_lig(a.GetName()) and a.GetName() not in self.modresidues]
+        else:
+            candidates2 = [a for a in candidates1 if is_lig(a.GetName())]
         write_message("%i ligand(s) after first filtering step.\n" % len(candidates2), mtype='debug')
 
         ############################################
@@ -788,7 +862,10 @@ class BindingSite(Mol):
         """Looks for positive charges in arginine, histidine or lysine, for negative in aspartic and glutamic acid."""
         data = namedtuple('pcharge', 'atoms atoms_orig_idx type center restype resnr reschain')
         a_set = []
-        for res in pybel.ob.OBResidueIter(mol.OBMol):
+        # Iterate through all residue, exclude those in chains defined as peptides
+        for res in [r for r in pybel.ob.OBResidueIter(mol.OBMol) if not r.GetChain() in config.PEPTIDES]:
+            if config.INTRA is not None:
+                if res.GetChain() != config.INTRA: continue
             a_contributing = []
             a_contributing_orig_idx = []
             if res.GetName() in ('ARG', 'HIS', 'LYS'):  # Arginine, Histidine or Lysine have charged sidechains
@@ -1152,6 +1229,10 @@ class PDBComplex:
         self.Mapper = Mapper()
         self.ligands = []
 
+    def __str__(self):
+        formatted_lig_names = [":".join([x.hetid, x.chain, str(x.position)]) for x in self.ligands]
+        return "Protein structure %s with ligands:\n" % (self.pymol_name) + "\n".join([lig for lig in formatted_lig_names])
+
     def load_pdb(self, pdbpath, as_string=False):
         """Loads a pdb file with protein AND ligand(s), separates and prepares them.
         If specified 'as_string', the input is a PDB string instead of a path."""
@@ -1224,6 +1305,11 @@ class PDBComplex:
         else:
             write_message("Structure contains no ligands.\n\n")
 
+    def analyze(self):
+        """Triggers analysis of all complexes in structure"""
+        for ligand in self.ligands:
+            self.characterize_complex(ligand)
+
     def characterize_complex(self, ligand):
         """Handles all basic functions for characterizing the interactions for one ligand"""
 
@@ -1235,6 +1321,10 @@ class PDBComplex:
         longname = ligand.longname if not len(ligand.longname) > 20 else ligand.longname[:20] + '...'
         ligtype = 'Unspecified type' if ligand.type == 'UNSPECIFIED' else ligand.type
         ligtext = "\n%s [%s] -- %s" % (longname, ligtype, site)
+        if ligtype == 'PEPTIDE':
+            ligtext = '\n Chain %s [PEPTIDE / INTER-CHAIN]' % ligand.chain
+        if ligtype == 'INTRA':
+            ligtext = "\n Chain %s [INTRA-CHAIN]" % ligand.chain
         any_in_biolip = len(set([x[0] for x in ligand.members]).intersection(config.biolip_list)) != 0
         write_message(ligtext)
         write_message('\n' + '-' * len(ligtext) + '\n')
@@ -1249,6 +1339,12 @@ class PDBComplex:
         bs_atoms = [self.atoms[idx] for idx in [i for i in self.atoms.keys()
                                                 if self.atoms[i].OBAtom.GetResidue().GetIdx() in bs_res]
                     if idx in self.Mapper.proteinmap and self.Mapper.mapid(idx, mtype='protein') not in self.altconf]
+        if ligand.type == 'PEPTIDE':
+            # If peptide, don't consider the peptide chain as part of the protein binding site
+            bs_atoms = [a for a in bs_atoms if a.OBAtom.GetResidue().GetChain() != lig_obj.chain]
+        if ligand.type == 'INTRA':
+            # Interactions within the chain
+            bs_atoms = [a for a in bs_atoms if a.OBAtom.GetResidue().GetChain() == lig_obj.chain]
         bs_atoms_refined = []
 
         # Create hash with BSRES -> (MINDIST_TO_LIG, AA_TYPE)
@@ -1277,9 +1373,14 @@ class PDBComplex:
         return [obres.GetIdx() for obres in resis if self.res_belongs_to_bs(obres, cutoff, ligcentroid)]
 
     def res_belongs_to_bs(self, res, cutoff, ligcentroid):
-        """Check for each residue if its centroid is within a certain distance to the ligand centroid."""
+        """Check for each residue if its centroid is within a certain distance to the ligand centroid.
+        Additionally checks if a residue belongs to a chain restricted by the user (e.g. by defining a peptide chain)"""
         rescentroid = centroid([(atm.x(), atm.y(), atm.z()) for atm in pybel.ob.OBResidueAtomIter(res)])
-        return True if euclidean3d(rescentroid, ligcentroid) < cutoff else False
+        # Check geometry
+        near_enough = True if euclidean3d(rescentroid, ligcentroid) < cutoff else False
+        # Check chain membership
+        restricted_chain = True if res.GetChain() in config.PEPTIDES else False
+        return (near_enough and not restricted_chain)
 
     def get_atom(self, idx):
         return self.atoms[idx]
