@@ -16,6 +16,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 from pymol import cmd
+from time import sleep
 import sys
 import os
 import subprocess
@@ -26,7 +27,9 @@ class PyMOLVisualizer:
         if not plcomplex is None:
             self.plcomplex = plcomplex
             self.protname = plcomplex.pdbid  # Name of protein with binding site
-            self.ligname = plcomplex.hetid  # Name of ligand
+            self.hetid = plcomplex.hetid
+            self.ligandtype = plcomplex.ligandtype
+            self.ligname = "Ligand_" + self.hetid  # Name of ligand
             self.metal_ids = plcomplex.metal_ids
 
     def set_initial_representations(self):
@@ -288,7 +291,7 @@ class PyMOLVisualizer:
 
         selections = cmd.get_names("selections")
         for selection in selections:
-            if len(cmd.get_model(selection).atom) == 0:
+            if cmd.count_atoms(selection) == 0:
                 cmd.delete(selection)
         cmd.deselect()
         cmd.delete('tmp*')
@@ -328,9 +331,11 @@ class PyMOLVisualizer:
                 cmd.zoom(self.ligname, 3)
         cmd.origin(self.ligname)
 
-    def save_session(self, outfolder):
+    def save_session(self, outfolder, override=None):
         """Saves a PyMOL session file."""
-        filename = '%s_%s' % (self.protname.upper(), "_".join([self.ligname, self.plcomplex.chain, self.plcomplex.position]))
+        filename = '%s_%s' % (self.protname.upper(), "_".join([self.hetid, self.plcomplex.chain, self.plcomplex.position]))
+        if override is not None:
+            filename = override
         cmd.save("/".join([outfolder, "%s.pse" % filename]))
 
     def png_workaround(self, filepath, width=1200, height=800):
@@ -397,6 +402,19 @@ class PyMOLVisualizer:
         cmd.set('ray_shadow', 0)  # Gives the molecules a flat, modern look
         cmd.set('ambient_occlusion_mode', 1)
 
+    def adapt_for_peptides(self):
+        """Adapt visualization for peptide ligands and interchain contacts"""
+        cmd.hide('sticks', self.ligname)
+        cmd.set('cartoon_color', 'lightorange', self.ligname)
+        cmd.show('cartoon', self.ligname)
+        cmd.show('sticks', "byres *-L")
+        cmd.util.cnc(self.ligname)
+        cmd.remove('%sCartoon and chain %s' % (self.protname, self.plcomplex.chain))
+        cmd.set('cartoon_side_chain_helper', 0)
+
+
+
+
     def refinements(self):
         """Refinements for the visualization"""
 
@@ -435,3 +453,6 @@ class PyMOLVisualizer:
 
         cmd.hide('everything', 'resn HOH &!Water')  # Hide all non-interacting water molecules
         cmd.hide('sticks', '%s and !%s and !AllBSRes' % (self.protname, self.ligname))  # Hide all non-interacting residues
+
+        if self.ligandtype in ['PEPTIDE', 'INTRA']:
+            self.adapt_for_peptides()
